@@ -21,7 +21,7 @@ Options:
               Random seed for reproducibility, if negative current time in second is applied. [default: -1]
 """
 
-__all__ = ['io','get_best','opt2md','opt2prms']
+__all__ = ['io','get_best','out2md','out2prms','db2csv','db2prms','individual']
 
 import os
 import sys
@@ -34,14 +34,14 @@ import subprocess
 import time
 from datetime import datetime
 
-from optzer.opt2prms import vars2params
+from optzer.out2prms import vars2params
 from optzer.io import read_in_optzer, write_info, write_vars_optzer, \
     read_vars_optzer, read_data
 from optzer.cs import CS
 from optzer.tpe import TPE
 
 __author__ = "RYO KOBAYASHI"
-__version__ = "0.2.2"
+__version__ = "0.2.7"
 
 _infname = 'in.optzer'
 
@@ -251,41 +251,51 @@ def func_wrapper(variables, **kwargs):
         #vars2params(varsfp['variables'], **kwargs)
 
     #...Do sub-jobs in the subdir_###
+    iid = kwargs['iid']
     L_up_lim = kwargs['fval_upper_limit']
+    subjob_done = True
     if print_level > 1:
         print('Performing subjobs at '+subdir, flush=True)
     try:
         prefix = kwargs['subjob-prefix']
         timeout= kwargs['subjob-timeout']
-        cmd = prefix +" ./{0:s} > log.iid_{1:d}".format(subjobscript,kwargs['iid'])
+        cmd = prefix +" ./{0:s} > log.iid_{1:d}".format(subjobscript,iid)
         subprocess.run(cmd,shell=True,check=True,timeout=timeout)
         optdata = get_data('.',**kwargs)
         L = loss_func(optdata,**kwargs)
 
-        #...Store data in iid_### directories
-        os.mkdir("iid_{0:d}".format(kwargs['iid']))
-        txt = ''
-        for t in kwargs['target']:
-            files = glob.glob(f'data*{t}')
-            if f'data.ref.{t}' in files:
-                files.remove(f'data.ref.{t}')
-            txt += ' {0:s}'.format(files[0])
-        for f in kwargs['param_files']:
-            txt += f' {f}'
-        for f in glob.glob('out.*'):
-            txt += f' {f}'
-        for f in glob.glob('*.yaml'):
-            text += f' {f}'
-        os.system("cp {0} iid_{1:d}/".format(txt,kwargs['iid']))
-        os.chdir(cwd)
     except Exception as e:
         if print_level > 0:
             print('  Since subjobs failed at {0:s}, '.format(subdir)
                   +'the upper limit value is applied to its loss function.',
                   flush=True)
         os.chdir(cwd)
+        subjob_done = False
         L = L_up_lim
 
+    if subjob_done:
+        try:
+            #...Store data in iid_### directories
+            os.mkdir("iid_{0:d}".format(iid))
+            txt = ''
+            for t in kwargs['target']:
+                files = glob.glob(f'data*{t}')
+                if f'data.ref.{t}' in files:
+                    files.remove(f'data.ref.{t}')
+                txt += ' {0:s}'.format(files[0])
+            for f in kwargs['param_files']:
+                txt += f' {f}'
+            for f in glob.glob('out.*'):
+                txt += f' {f}'
+            for f in glob.glob('*.yaml'):
+                txt += f' {f}'
+            os.system("cp {0} iid_{1:d}/".format(txt,iid))
+            os.chdir(cwd)
+        except Exception as e:
+            if print_level > 0:
+                print('  Something wrong with storing data for iid_{0:d}.'.format(iid))
+                print('    Error: ', e)
+            pass
         
     return L
     
@@ -359,7 +369,7 @@ def main():
     #               write_vars_optzer, seed=seed, **kwargs)
     # opt.run(maxiter)
 
-    print('\n optzer finished since it exceeds the max interation.')
+    # print('\n optzer finished since it exceeds the max interation.')
     print(' Elapsed time = {0:.1f} sec.'.format(time.time()-start))
     
     return None
@@ -375,7 +385,7 @@ def headline():
     print('            at {0:s}'.format(datetime.now().strftime('%Y-%m-%d %H:%M:%S')))
     print()
     print('   Please cite:')
-    print('     1) R. Kobayashi, J. Open Source Software, 6(57), 2768 (2021)')
+    print('     1) github.com/ryokbys/optzer')
     print()
     return None
 
