@@ -21,7 +21,6 @@ import pandas as pd
 from optzer.individual import Individual, ind_from_db, update_slims
 from optzer.io import write_db_optzer, read_db_optzer
 
-
 __author__ = "RYO KOBAYASHI"
 __version__ = "221228"
 
@@ -39,7 +38,8 @@ class TPE:
     """
 
     def __init__(self, nbatch, vnames, vs0, slims, hlims, loss_func,
-                 write_func=None, seed=42, **kwargs):
+                 write_func=None, seed=42, criterion=-1.0,
+                 **kwargs):
         """
         Conctructor of TPE class.
 
@@ -56,6 +56,9 @@ class TPE:
               Loss function to be minimized with variables and **kwargs.
           write_func : function
               Function for outputing some info.
+          criterion: float
+              Convergence criterion for the loss.
+              If negtive (default), not to set criterion.
         """
         if nbatch < 1:
             raise ValueError('nbatch must be > 0.')
@@ -66,13 +69,14 @@ class TPE:
         self.vnames = vnames
         self.vs0 = vs0
         self.hlims = hlims
-        #self.slims = slims
-        self.slims = hlims  # Since slims is not used, use hlims instead as slims
+        self.slims = slims
+        self.hlims = hlims
         self.vlogs = { k:False for k in vnames }
         if 'vlogs' in kwargs.keys():
             self.vlogs = kwargs['vlogs']
         self.loss_func = loss_func
         self.write_func = write_func
+        self.criterion = criterion
         self.kwargs = kwargs
         self.best_pnt = None
         self.print_level = 0
@@ -145,6 +149,7 @@ class TPE:
                 if i == 0:
                     smpl.set_variables(self.vs0, self.slims)
                 else:
+                    #...Random pick in soft bounds
                     smpl.init_random(self.slims)
                 self.candidates.append(smpl)
         else:  # DB loaded
@@ -187,6 +192,19 @@ class TPE:
 
         starttime = time()
 
+        #...Whether of not kwargs contains 'temperature'...
+        if 'initial_temperature' in self.kwargs.keys():
+            tini = self.kwargs['initial_temperature']
+            tfin = self.kwargs['final_temperature']
+            self.kwargs['temperature'] = tini
+            dtemp = (tfin -tini) /max(maxstp,1)
+            print('   Initial,final,delta temperatures = '
+                  + f'{tini:.1f}  {tfin:.1f}  {dtemp:.3f}')
+            print('',flush=True)
+        elif 'temperature' in self.kwargs.keys():
+            print('   Temperature = {0:.1f}'.format(self.kwargs['temperature']))
+            print('',flush=True)
+            
         #...Create pool before going into maxstp-loop,
         #...since creating pool inside could cause "Too many files" error.
         pool = Pool(processes=self.nbatch)
@@ -225,6 +243,14 @@ class TPE:
         if self.print_level > 0:
             self._write_step_info(self.igen0,starttime)
 
+        if self.criterion > 0.0:
+            if self.bestsmpl.loss < self.criterion:
+                print(' Convergence achieved since the best loss < criterion.\n'
+                      +'   Best loss and criterion = '
+                      +'{0:.3f}  {1:.3f}'.format(self.bestsmpl.loss,
+                                                 self.criterion))
+                return None
+
         #...TPE loop starts
         for igen in range(self.igen0+1, self.igen0+1+maxstp):
             #...Create candidates by either random or TPE
@@ -238,8 +264,13 @@ class TPE:
                     self.candidates.append(newsmpl)
             else:
                 if self.method in ('wpe,''WPE'):
+                    temp = 1.0
+                    if 'temperature' in self.kwargs.keys():
+                        temp = self.kwargs['temperature']
                     #...Create candidates by WPE
-                    self.candidates = self._candidates_by_WPE()
+                    self.candidates = self._candidates_by_WPE(temperature=temp)
+                    if 'initial_temperature' in self.kwargs.keys():
+                        self.kwargs['temperature'] += dtemp
                 else:
                     #...Create candidates by TPE
                     self.candidates = self._candidates_by_TPE()
@@ -284,7 +315,16 @@ class TPE:
             if self.print_level > 0:
                 self._write_step_info(igen,starttime)
             
+            if self.criterion > 0.0:
+                if self.bestsmpl.loss < self.criterion:
+                    print(' Convergence achieved since the best loss < criterion.\n'
+                          +'   Best loss and criterion = '
+                          +'{0:.3f}  {1:.3f}'.format(self.bestsmpl.loss,
+                                                     self.criterion))
+                    return None
+
         pool.close()
+        print(' Finished since it exceeds the max iteration')
         return None
 
     def _write_step_info(self,istp,starttime):
@@ -311,23 +351,29 @@ class TPE:
         nlow = int(self.gamma *len(losses))
         nhigh = len(losses) -nlow
         argpart = np.argpartition(losses,nlow)
+        argsrt = np.argsort(losses)
         Xlow = np.zeros((nlow,self.ndim))
         Xhigh = np.zeros((nhigh,self.ndim))
         for i in range(nlow):
-            idx = argpart[i]
+            #idx = argpart[i]
+            idx = argsrt[i]
             vs = ind_from_db(self.history_db,idx,self.vnames,self.slims).vs
             for j,k in enumerate(self.vnames):
                 Xlow[i,j] = vs[k]
         for i in range(nhigh):
-            idx = argpart[nlow+i]
+            #idx = argpart[nlow+i]
+            idx = argsrt[nlow+i]
             vs = ind_from_db(self.history_db,idx,self.vnames,self.slims).vs
             for j,k in enumerate(self.vnames):
                 Xhigh[i,j] = vs[k]
+        
 
         #...Sampling variable candidates
         xcandidates = np.empty((self.nbatch,self.ndim))
         ntrial = max(self.ntrial,self.nbatch)
+        # print('')
         for idim in range(self.ndim):
+            # print('idim = ',idim)
             key = self.vnames[idim]
             # xhmin = self.hlims[key][0]
             # xhmax = self.hlims[key][1]
@@ -340,11 +386,16 @@ class TPE:
             std = np.std(xlowsrt)
             sgm = min(std, (q75-q25)/1.34)
             h = 1.06 *sgm /np.power(npnt, 1.0/5)
+            # print(f'  q25,q75,std,sgm,h= {q25:.3f}, {q75:3f},'
+            #       +f' {std:.3f}, {sgm:.3f}, {h:.3f}')
+            # print('  xlowsrt=',xlowsrt)
             #...Prepare for g(x)
             xhighsrt = Xhigh[:,idim]
             q75, q25 = np.percentile(xhighsrt, [75,25])
             sgmh = min(np.std(xhighsrt), (q75-q25)/1.34)
             hh = 1.06 *sgmh /np.power(len(xhighsrt),1.0/5)
+            # print(f'  q25,q75,sgmh,hh= {q25:.3f}, {q75:3f},'
+            #       +f' {sgmh:.3f}, {hh:.3f}')
             #...Several trials for selection
             aquisition = np.zeros(ntrial)
             xs = np.empty(ntrial)
@@ -372,6 +423,8 @@ class TPE:
             #...Pick nbatch of minimum aquisition points
             idxsort = np.argsort(aquisition)
             xcandidates[:,idim] = xs[idxsort[0:self.nbatch]]
+            # print('  aquisition=',aquisition[idxsort[0:self.nbatch]])
+            # print('  xcandidates=',xcandidates[:,idim])
         #...Create sample with xcandidate as variables
         candidates = []
         for ib in range(self.nbatch):
@@ -384,7 +437,7 @@ class TPE:
             candidates.append(smpl)
         return candidates
 
-    def _candidates_by_WPE(self,):
+    def _candidates_by_WPE(self,veps=1e-8,temperature=1.0):
         """
         Create candidates by using WPE.
         """
@@ -399,8 +452,10 @@ class TPE:
             #tmpsmpls = copy.copy(self.history)
             tmpsmpls = [ ind_from_db(self.history_db,idx,self.vnames,self.slims)
                          for idx in self.history_db.index ]
+
         vmin = losses.min()
-        wgts = np.array([ np.exp(-(v-vmin)/vmin) for v in losses ])
+        wgts = np.array([ np.exp(-(v-vmin)/(vmin+veps)/temperature)
+                          for v in losses ])
         xtmps = np.zeros((len(tmpsmpls),self.ndim))
         for i in range(len(tmpsmpls)):
             for j,k in enumerate(self.vnames):
